<%!
    import datetime
    from dateutil import tz

    def relative_time(timestamp):
      seconds = (datetime.datetime.now(tz.UTC) - datetime.datetime.fromisoformat(timestamp)).seconds

      days = hours = minutes = 0
      if seconds > 86400:
        days, seconds = divmod(seconds, 86400)
      if seconds > 3600:
        hours, seconds = divmod(seconds, 3600)
      if seconds > 60:
        minutes, seconds = divmod(seconds, 60)

      return f"{days}d, {hours:02d}:{minutes:02d}:{seconds:02d}"
%>

<%def name="cachedata(stream_list)">
  <h3>Stream Cache <a title="This is a list of every stream that the middleware thinks is live. It should match the list of live streams reported by the OvenMediaEngine API above. If they don't match, there's a bug in OvenEmprex's cache handling and some streams may not appear properly."><u>[?]</u></a></h3>
  <ul>
  % if not stream_list:
    <li>No Live Streams</li>
  % endif
  % for stream in stream_list:
    <li>${stream[0]}:${stream[1]}:${stream[2]}</li>
  % endfor
  </ul>
</%def>

<%def name="blockdata(disabled, blocked_ips, blocked_keys)">
  % if disabled:
    <h2 class="warning">The server is currently disabled!</h2>
  % endif

  % if blocked_ips:
    <details open><summary>Blocked IPs</summary>
    <ul>
    % for ip in blocked_ips:
      <li>${ip} <button onclick="location.href='/management/unban?target=${ip}';">Unblock</button></li>
    % endfor
    </ul>
    </details>
  % endif

  % if blocked_keys:
    <details open><summary>Disabled Keys</summary>
    <ul>
    % for streamkey in blocked_keys:
      <li>${streamkey} <button onclick="location.href='/management/enable?target=${streamkey}';">Re-enable</button></li>
    % endfor
    </ul>
    </details>
  % endif

  % if disabled or blocked_ips or blocked_keys:
    <hr>
  % endif
</%def>

<%def name="stream_buttons(vhost, app, stream)">
  <button onclick="location.href='/management/disconnect?target=${vhost}:${app}:${stream}';">Disconnect</button>
  <button onclick="location.href='/management/ban?target=${vhost}:${app}:${stream}';">Ban IP</button>
  <button onclick="location.href='/management/disable?target=${vhost}:${app}:${stream}';">Disable Key</button>
</%def>

<html>

<head>
<style>
body {
  background-color: #000;
  color: #ddd;
}
details {
  margin-left: 1rem;
}
summary {
  margin-left: -1rem;
  margin-top: 5px;
  font-weight: bold;
}
.warning {
  color: #ff0;
}
.alert {
  font-weight: bold;
  color: #f00;
}
td {
  text-align: right;
  background: #111;
}
td.left {
  text-align: left;
}
</style>
</head>

<body>
  <button type="submit" onclick="location.href='/management/restart';">Restart Server</button>
  <button type="submit" onclick="location.href='/management/stop';">Disable Server</button>
  <button type="submit" onclick="location.href='/management/start';">Re-Enable Server</button>
  <hr>
  ${blockdata(DISABLED, BLOCKED_IPS, DISABLED_KEYS)}

  % for vhost_name, vhost_data in data["vhosts"].items():
    <details open><summary>${vhost_name} vhost</summary>
    <table>
    % for vhost_k, vhost_v in vhost_data.items():
      % if vhost_k != "apps":
        <tr><td>${vhost_k}</td><td>${vhost_v}</td></tr>
      % endif      
    % endfor
    </table>

    % for app_name, app_data in vhost_data.get("apps", {}).items():
      <details open><summary>${app_name}</summary>
      <table>
      % for app_k, app_v in app_data.items():
        % if app_k != "streams":
          <tr><td>${app_k}</td><td>${app_v}</td></tr>
        % endif      
      % endfor
      </table>

      % for stream_name, stream_data in app_data.get("streams", {}).items():
        <details open><summary>${app_name}/${stream_name}</summary>
        <table>
        % for stream_k, stream_v in stream_data.items():
          <%
            if stream_k == "has_bframes" and stream_v:
              classname = 'class="alert"'
            else:
              classname = ''

            if stream_k == "fps_actual":
              stream_v = f"{stream_v:.1f}"
            elif stream_k.startswith("bitrate_"):
              stream_v = f"{stream_v / 1000:.0f}kbps"
            elif stream_k == "created":
              stream_v = f"{relative_time(stream_v)} ago"
          %>
          <tr><td class='left'>${stream_k}</td><td ${classname}>${stream_v}</td></tr>
        % endfor
        </table>
        ${stream_buttons(vhost_name, app_name, stream_name)}
        </details>
      % endfor
      </details>
    % endfor
    </details>
  % endfor
  <hr>
  <details><summary>Debug Info</summary>
    ${cachedata(STREAM_LIST)}
  </details>
</body>
<html>
